"""
Author: "Rangana Warshamanage, Garib N. Murshudov"
MRC Laboratory of Molecular Biology
    
This software is released under the
Mozilla Public License, version 2.0; see LICENSE.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import numpy as np
import os
from timeit import default_timer as timer
import numpy as np
import fcodes_fast
from emda.iotools import read_map,write_mrc,resample2staticmap
from emda.mapfit.utils import remove_unwanted_corners
import emda.plotter
from emda.config import *
import emda.mapfit.utils as utils
from emda import fsc

#debug_mode = 1 # 0: no debug info, 1: debug

class EmmapOverlay:
    
    def __init__(self,hfmap_list, mask_list=None):
        self.hfmap_list         = hfmap_list
        self.mask_list          = mask_list
        self.map_unit_cell      = None
        self.map_origin         = None
        self.map_dim            = None
        self.arr_lst            = []
        #
        self.ceo_lst            = None
        self.cfo_lst            = None
        self.cbin_idx           = None
        self.cdim               = None
        self.cbin               = None

    def load_maps(self, fobj):
        from scipy import ndimage
        from scipy.ndimage.interpolation import shift
        #import fcodes_fast
        # read mask and map
        fhf_lst = []
        if self.mask_list is not None:
            if len(self.hfmap_list) != len(self.mask_list):
                print('map_list and mask_list must have the same size!')
                print('exiting program...')
                exit()
            for i in range(len(self.mask_list)):
                _,mask,_ = read_map(self.mask_list[i])
                mask = utils.set_dim_even(mask)
                uc,arr,origin = read_map(self.hfmap_list[i])
                arr = utils.set_dim_even(arr)
                if i == 0: 
                    temp = arr * mask
                    com1 = ndimage.measurements.center_of_mass(temp * temp)
                    print('COM: ', com1)
                    nx, ny, nz = arr.shape
                    box_centr = (nx//2, ny//2, nz//2)
                    print(box_centr)
                    self.com1 = com1
                    self.box_centr = box_centr
                    map_origin = origin
                    uc_target = uc
                    target_dim = arr.shape
                    target_pix_size = uc_target[0]/target_dim[0]
                    corner_mask = remove_unwanted_corners(uc,target_dim)
                    arr_mask = shift(arr * mask, np.subtract(box_centr,com1))

                    fhf_lst.append(np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr_mask * corner_mask))))
                else:
                    mask = resample2staticmap(target_pix_size,target_dim,uc,mask)
                    arr = resample2staticmap(target_pix_size,target_dim,uc,arr) 
                    temp = arr * mask               
                    com1 = ndimage.measurements.center_of_mass(temp * temp) 
                    print('COM: ', com1)            
                    arr_mask = shift(arr * mask, np.subtract(box_centr,com1))                  
                    fhf_lst.append(np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr_mask * corner_mask))))

            # free memory
            del arr, arr_mask, mask
            del corner_mask
            #
            self.map_origin     = map_origin
            self.map_unit_cell  = uc_target
            self.map_dim        = target_dim 
            self.fhf_lst        = fhf_lst 

        if self.mask_list is None: 
            for i in range(len(self.hfmap_list)):
                uc,arr,origin = read_map(self.hfmap_list[i])
                arr = utils.set_dim_even(arr)
                print('origin: ', origin)
                if i == 0:
                    #write_mrc(arr,'static.mrc',uc)
                    #uc,arr,origin = read_map('static.mrc')
                    com1 = ndimage.measurements.center_of_mass(arr * (arr >= 0.)) # to remove effect from negative values
                    print('COM: ', com1)
                    nx, ny, nz = arr.shape
                    box_centr = (nx//2, ny//2, nz//2)
                    print('BOX center: ', box_centr)
                    self.com1 = com1
                    self.box_centr = box_centr
                    map_origin = origin
                    uc_target = uc
                    target_dim = arr.shape
                    target_pix_size = uc_target[0]/target_dim[0]
                    arr = shift(arr, np.subtract(box_centr,com1)) 
                    self.arr_lst.append(arr)
                    write_mrc(arr,'static_centered.mrc',uc_target,map_origin)
                    print('COM after centering: ', ndimage.measurements.center_of_mass(arr * (arr >= 0.)))
                    fhf_lst.append(np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr))))
                else:
                    arr = resample2staticmap(target_pix_size,target_dim,uc,arr)
                    com1 = ndimage.measurements.center_of_mass(arr * (arr >= 0.)) # to remove effect from negative values
                    #com1 = ndimage.measurements.center_of_mass(arr * arr )
                    print('COM: ', com1)
                    arr = shift(arr, np.subtract(box_centr,com1))
                    self.arr_lst.append(arr)
                    write_mrc(arr,'moving_centered.mrc',uc_target,map_origin) 
                    print('COM after centering: ', ndimage.measurements.center_of_mass(arr * (arr >= 0.)))
                    fhf_lst.append(np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr))))
                
            # free memory
            del arr
            #            
            self.map_origin     = map_origin
            self.map_unit_cell  = uc_target
            self.map_dim        = target_dim 
            self.fhf_lst        = fhf_lst 

    def calc_fsc_from_maps(self, fobj):  
        # function for only two maps fitting
        import fcodes_fast
        from emda import restools
        nmaps = len(self.fhf_lst) 
        fFo_lst = []
        fEo_lst = []
        fBTV_lst = []
        #
        nx,ny,nz = self.fhf_lst[0].shape
        self.nbin,self.res_arr,self.bin_idx = restools.get_resolution_array(self.map_unit_cell,
                                                                       self.fhf_lst[0])
        #
        for i in range(nmaps):  
            _,_,_,totalvar,fo,eo = fsc.halfmaps_fsc_variance(self.fhf_lst[i],
                                                             self.fhf_lst[i],
                                                             self.bin_idx,
                                                             self.nbin)
            #fo,eo,_,_,totalvar,_ = fcodes_fast.calc_fsc_using_halfmaps(
            #    self.fhf_lst[i],self.fhf_lst[i],self.bin_idx,self.nbin,debug_mode,nx,ny,nz)  
            fFo_lst.append(fo)
            fEo_lst.append(eo)
            fBTV_lst.append(totalvar)
        #
        self.fo_lst            = fFo_lst
        self.eo_lst            = fEo_lst
        self.totalvar_lst      = fBTV_lst



class EmmapAverage:
    
    def __init__(self,hfmap_list,mask_list=None):
        self.hfmap_list         = hfmap_list
        self.mask_list          = mask_list
        self.map_unit_cell      = None
        self.map_origin         = None
        self.map_dim            = None
        self.com_lst            = None
        self.resol_rand         = 10.0 # resolution (A) for phase randomisation
        #
        self.ceo_lst            = None
        self.cbin_idx           = None
        self.cdim               = None
        self.cbin               = None

    def load_maps(self, fobj):
        from scipy import ndimage
        from scipy.ndimage.interpolation import shift
        from emda.fsc_true_from_phase_randomize import get_randomized_sf
        import fcodes_fast
        # read masks
        fhf_lst = []
        unmask_fhf_lst = []
        phrand_fhf_lst = []
        com_lst = []
        if self.mask_list is not None:
            if len(self.hfmap_list) != len(self.mask_list):
                print('hfmap_list and mask_list must have the same size!')
                print('exiting program...')
                exit()
            '''mask_lst = []
            fobj.write('\n Masks files:\n')
            for i in range(0,len(self.mask_list),2):
                fobj.write('%s\n' %os.path.abspath(self.mask_list[i]))
                _,mask,_ = read_map(self.mask_list[i])
                mask_lst.append(mask) # for hfmap1 & hfmap2
            fobj.write('\n Input files (static map):\n') '''
            for i in range(0,len(self.hfmap_list),2):
                '''fobj.write('\n Masks files:\n')
                fobj.write('%s\n' %os.path.abspath(self.mask_list[i]))
                _,mask,_ = read_map(self.mask_list[i])
                mask_lst.append(mask) # for hfmap1 & hfmap2
                fobj.write('\n Input files (static map):\n')
                uc,arr1,origin = read_map(self.hfmap_list[i])
                fobj.write('%s\n' %os.path.abspath(self.hfmap_list[i]))
                uc,arr2,origin = read_map(self.hfmap_list[i+1])
                fobj.write('%s\n' %os.path.abspath(self.hfmap_list[i+1]))'''
                if i == 0: 
                    fobj.write('Static map:\n')
                    fobj.write('Mask file: %s\n' %os.path.abspath(self.mask_list[i]))
                    _,mask,_ = read_map(self.mask_list[i]) # for hfmap1 & hfmap2
                    fobj.write('Input files (static map):\n')
                    uc,arr1,origin = read_map(self.hfmap_list[i])
                    fobj.write('%s\n' %os.path.abspath(self.hfmap_list[i]))
                    uc,arr2,origin = read_map(self.hfmap_list[i+1])
                    fobj.write('%s\n' %os.path.abspath(self.hfmap_list[i+1]))
                    com1 = ndimage.measurements.center_of_mass(arr1 * mask)#mask_lst[0])
                    com_lst.append(com1)
                    nx, ny, nz = arr1.shape
                    box_centr = (nx//2, ny//2, nz//2)
                    print(box_centr)
                    fobj.write('Center_of_mass coordinates: ' + str(com1) + ' \n')
                    self.box_centr = box_centr
                    fobj.write('Center_of_box coordinates: ' + str(box_centr) + '\n')
                    map_origin = origin
                    fobj.write('Map origin coordinates: ' + str(map_origin) + ' \n')
                    uc_target = uc
                    fobj.write('Unit cell: ' + str(uc_target) + ' \n')
                    target_dim = arr1.shape
                    fobj.write('Map dimensions: ' + str(target_dim) + ' \n')
                    target_pix_size = uc_target[0]/target_dim[0]
                    fobj.write('Map pixel size: ' + str(target_pix_size) + ' \n')
                    corner_mask = remove_unwanted_corners(uc,target_dim)
                    # get resolution grid
                    maxbin = np.amax(np.array([nx//2,ny//2,nz//2]))
                    fobj.write('Creating resolution grid... \n')
                    resol_grid, self.s_grid, _ = fcodes_fast.resolution_grid_full(uc,0.0,1,maxbin,nx,ny,nz)
                    #
                    fobj.write('Phase randomization using static half maps. \n')
                    fobj.write('Phase randomize resolution:'+str(self.resol_rand)+' \n')
                    #test_hflist = []
                    for arr in [arr1, arr2]:
                        arr_unmask = arr
                        fhf1_randomized = get_randomized_sf(resol_grid,arr_unmask,self.resol_rand)
                        arr_rand = np.real(np.fft.ifftn(np.fft.ifftshift(fhf1_randomized))) * mask
                        arr_mask = shift(arr * mask, np.subtract(box_centr,com1))
                        # test
                        #write_mrc(arr_mask,'test.mrc',uc_target,map_origin)
                        #fmask = np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr_mask * corner_mask)))
                        #test_hflist.append(fmask)
                        #data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(fmask))))
                        #data2write = shift(data2write, np.subtract(com_lst[0],self.box_centr))
                        #write_mrc(data2write,'test2.mrc',uc_target,map_origin)
                        #exit()
                        #
                        arr_unmask = shift(arr_unmask, np.subtract(box_centr,com1)) 
                        arr_rand = shift(arr_rand, np.subtract(box_centr,com1))
                        fhf_lst.append(np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr_mask * corner_mask))))
                        unmask_fhf_lst.append(np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr_unmask * 
                                                                                          corner_mask))))
                        phrand_fhf_lst.append(np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr_rand * 
                                                                                          corner_mask))))
                    '''fullmap_stat = (test_hflist[0] + test_hflist[0])/2.0
                    data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(fullmap_stat))))
                    data2write = shift(data2write, np.subtract(com_lst[0],self.box_centr))
                    write_mrc(data2write,'test3.mrc',uc_target,map_origin)
                    exit()'''
                else:
                    fobj.write('Moving map:\n')
                    fobj.write('Mask file: %s\n' %os.path.abspath(self.mask_list[i]))
                    #fobj.write('%s\n' %os.path.abspath(self.mask_list[i]))
                    _,mask,_ = read_map(self.mask_list[i]) # for hfmap1 & hfmap2
                    fobj.write('Resampling mask...\n')
                    mask = resample2staticmap(target_pix_size,target_dim,uc,mask,fobj=fobj)
                    #mask_lst.append(mask) # for hfmap1 & hfmap2
                    fobj.write('Input files (moving map):\n')
                    uc,arr1,origin = read_map(self.hfmap_list[i])
                    fobj.write('%s\n' %os.path.abspath(self.hfmap_list[i]))
                    uc,arr2,origin = read_map(self.hfmap_list[i+1])
                    fobj.write('%s\n' %os.path.abspath(self.hfmap_list[i+1]))
                    run_once = True
                    #mask = resample2staticmap(target_pix_size,target_dim,uc,mask) #mask_lst[i//2])
                    fobj.write('Phase randomization using half maps. \n')
                    fobj.write('Phase randomize resolution:'+str(self.resol_rand)+' \n')
                    for arr in [arr1, arr2]:
                        fobj.write('Resampling halfmaps...\n')
                        arr = resample2staticmap(target_pix_size,target_dim,uc,arr,fobj=fobj)
                        arr_unmask = arr
                        fobj.write('Phase randomization...\n')
                        fhf1_randomized = get_randomized_sf(resol_grid,arr_unmask,self.resol_rand)
                        arr_rand = np.real(np.fft.ifftn(np.fft.ifftshift(fhf1_randomized))) * mask
                        if run_once: 
                            com1 = ndimage.measurements.center_of_mass(arr * mask)
                            com_lst.append(com1)
                        run_once = False
                        arr_mask = shift(arr * mask, np.subtract(box_centr,com1))
                        arr_unmask = shift(arr_unmask, np.subtract(box_centr,com1)) 
                        arr_rand = shift(arr_rand, np.subtract(box_centr,com1))
                        fhf_lst.append(np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr_mask * corner_mask))))
                        unmask_fhf_lst.append(np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr_unmask * 
                                                                                          corner_mask))))
                        phrand_fhf_lst.append(np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr_rand * 
                                                                                          corner_mask))))

            # free memory
            #del mask_lst
            del arr, arr1, arr2
            del corner_mask
            #
            self.map_origin     = map_origin
            self.map_unit_cell  = uc_target
            self.map_dim        = target_dim 
            self.fhf_lst        = fhf_lst 
            self.com_lst        = com_lst
            self.unmask_fhf_lst = unmask_fhf_lst
            self.phrand_fhf_lst = phrand_fhf_lst

        if self.mask_list is None: 
            print('Correlation based masks will be generated and used for fitting!')
            from emda import maskmap_class
            obj_maskmap = maskmap_class.MaskedMaps()
            for i in range(0,len(self.hfmap_list),2):
                uc,arr1,origin = read_map(self.hfmap_list[i])
                uc,arr2,origin = read_map(self.hfmap_list[i+1])
                # calculate the mask
                obj_maskmap.generate_mask(arr1, arr2)
                mask = obj_maskmap.mask
                write_mrc(mask,"{0}_{1}.{2}".format('mask',str(i),'mrc'),uc,origin)
                if i == 0:
                    com1 = ndimage.measurements.center_of_mass(arr1 * mask)
                    nx, ny, nz = arr1.shape
                    box_centr = (nx//2, ny//2, nz//2)
                    print(box_centr)
                    self.com1 = com1
                    self.box_centr = box_centr
                    map_origin = origin
                    uc_target = uc
                    target_dim = arr1.shape
                    target_pix_size = uc_target[0]/target_dim[0]
                    # get resolution grid
                    maxbin = np.amax(np.array([nx//2,ny//2,nz//2]))
                    resol_grid, self.s_grid, _ = fcodes_fast.resolution_grid_full(uc,0.0,1,maxbin,nx,ny,nz)
                    #
                    for arr in [arr1, arr2]:
                        arr_unmask = arr
                        fhf1_randomized = get_randomized_sf(resol_grid,arr_unmask,self.resol_rand)
                        arr_rand = np.real(np.fft.ifftn(np.fft.ifftshift(fhf1_randomized))) * mask
                        arr_mask = shift(arr * mask, np.subtract(box_centr,com1))
                        arr_unmask = shift(arr_unmask, np.subtract(box_centr,com1)) 
                        arr_rand = shift(arr_rand, np.subtract(box_centr,com1))
                        fhf_lst.append(np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr_mask))))
                        unmask_fhf_lst.append(np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr_unmask))))
                        phrand_fhf_lst.append(np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr_rand))))
                else:
                    run_once = True
                    for arr in [arr1, arr2]:
                        mask = resample2staticmap(target_pix_size,target_dim,uc,mask)
                        arr = resample2staticmap(target_pix_size,target_dim,uc,arr)
                        arr_unmask = arr
                        fhf1_randomized = get_randomized_sf(resol_grid,arr_unmask,self.resol_rand)
                        arr_rand = np.real(np.fft.ifftn(np.fft.ifftshift(fhf1_randomized))) * mask
                        if run_once: com1 = ndimage.measurements.center_of_mass(arr * mask)
                        run_once = False
                        arr_mask = shift(arr * mask, np.subtract(box_centr,com1))
                        arr_unmask = shift(arr_unmask, np.subtract(box_centr,com1)) 
                        arr_rand = shift(arr_rand, np.subtract(box_centr,com1))
                        fhf_lst.append(np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr_mask))))
                        unmask_fhf_lst.append(np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr_unmask))))
                        phrand_fhf_lst.append(np.fft.fftshift(np.fft.fftn(np.fft.fftshift(arr_rand))))
                
            # free memory
            del arr, arr1, arr2
            #            
            self.map_origin     = map_origin
            self.map_unit_cell  = uc_target
            self.map_dim        = target_dim 
            self.fhf_lst        = fhf_lst 
            self.unmask_fhf_lst = unmask_fhf_lst 
            self.phrand_fhf_lst = phrand_fhf_lst

    def calc_fsc_variance_from_halfdata(self, fobj): 
        import fcodes_fast 
        from emda import restools
        from scipy.ndimage.interpolation import shift
        nmaps = len(self.fhf_lst) 
        fFo_lst = []
        fEo_lst = []
        fBNV_lst = []
        fBSV_lst = []
        fBTV_lst = []
        fBFsc_lst = []
        umfFo_lst = []
        umfEo_lst = []
        umfBNV_lst = []
        umfBSV_lst = []
        umfBTV_lst = []
        umfBFsc_lst = []
        #
        fobj.write('\n Calculating Fourier Shell Correlation using half data \n')
        fobj.write('   ----------------------------------------------------- \n')
        fobj.write('\n')
        nx,ny,nz = self.fhf_lst[0].shape
        self.nbin,self.res_arr,self.bin_idx = restools.get_resolution_array(self.map_unit_cell,
                                                                       self.fhf_lst[0])
        idx = np.argmin((self.res_arr - self.resol_rand)**2)
        #
        for i in range(0,nmaps,2): 
            # unmasked data
            fobj.write('Unmasked maps %s\n' %os.path.abspath(self.hfmap_list[i]))
            fobj.write('              %s\n' %os.path.abspath(self.hfmap_list[i+1]))
            fobj.write('\n')
            umbin_fsc,umnoisevar,umsignalvar,umtotalvar,umfo,umeo = fsc.halfmaps_fsc_variance(
                                                             self.unmask_fhf_lst[i],
                                                             self.unmask_fhf_lst[i+1],
                                                             self.bin_idx,
                                                             self.nbin)
            #umfo,umeo,umnoisevar,umsignalvar,umtotalvar,umbin_fsc = fcodes_fast.calc_fsc_using_halfmaps(
            #    self.unmask_fhf_lst[i],self.unmask_fhf_lst[i+1],self.bin_idx,self.nbin,debug_mode,nx,ny,nz)
            data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(umfo))))
            data2write = shift(data2write, np.subtract(self.com_lst[0],self.box_centr))
            write_mrc(data2write,'test4.mrc',self.map_unit_cell,self.map_origin)
            fobj.write(' bin# \n')
            fobj.write(' bin resolution (A) \n')
            fobj.write(' Signal Variance \n')
            fobj.write(' Noise variance \n')
            fobj.write(' Total variance \n')
            fobj.write(' Halfmap FSC \n')
            fobj.write('\n')
            for j in range(len(self.res_arr)):
                fobj.write("{:5d} {:6.2f} {:8.4f} {:8.4f} {:8.4f} {:8.4f}\n".format(
                            j,
                            self.res_arr[j],
                            umsignalvar[j],
                            umnoisevar[j],
                            umtotalvar[j],
                            umbin_fsc[j]))
            full_fsc_unmasked = 2.0 * umbin_fsc / (1.0 + umbin_fsc)
            # masked data
            fobj.write('\n')
            fobj.write('Masked maps %s\n' %os.path.abspath(self.hfmap_list[i]))
            fobj.write('            %s\n' %os.path.abspath(self.hfmap_list[i+1]))
            fobj.write('\n')
            bin_fsc,noisevar,signalvar,totalvar,fo,eo = fsc.halfmaps_fsc_variance(self.fhf_lst[i],
                                                             self.fhf_lst[i+1],
                                                             self.bin_idx,
                                                             self.nbin)
            #fo,eo,noisevar,signalvar,totalvar,bin_fsc = fcodes_fast.calc_fsc_using_halfmaps(
            #    self.fhf_lst[i],self.fhf_lst[i+1],self.bin_idx,self.nbin,debug_mode,nx,ny,nz) 
            # test
            data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(fo))))
            data2write = shift(data2write, np.subtract(self.com_lst[0],self.box_centr))
            write_mrc(data2write,'test5.mrc',self.map_unit_cell,self.map_origin)
            exit()
            #
            fobj.write(' bin# \n')
            fobj.write(' bin resolution (A) \n')
            fobj.write(' Signal Variance \n')
            fobj.write(' Noise variance \n')
            fobj.write(' Total variance \n')
            fobj.write(' Halfmap FSC \n')
            fobj.write('\n')
            for j in range(len(self.res_arr)):
                fobj.write("{:5d} {:6.2f} {:8.4f} {:8.4f} {:8.4f} {:8.4f}\n".format(
                            j,
                            self.res_arr[j],
                            signalvar[j],
                            noisevar[j],
                            totalvar[j],
                            bin_fsc[j]))
            full_fsc_total = 2.0 * bin_fsc / (1.0 + bin_fsc)
            #weighted_grid = fcodes_fast.read_into_grid(self.bin_idx,bin_fsc,self.nbin,nx,ny,nz)
            #data2write = np.real(np.fft.ifftshift(np.fft.ifftn(np.fft.ifftshift(eo * weighted_grid))))
            #write_mrc(data2write,'weightedmap.mrc',self.map_unit_cell)
            #exit()
            # randomised data
            rbin_fsc,_,_,_,_,_ = fsc.halfmaps_fsc_variance(self.phrand_fhf_lst[i],
                                                             self.phrand_fhf_lst[i+1],
                                                             self.bin_idx,
                                                             self.nbin)
            #_,_,_,_,_,rbin_fsc = fcodes_fast.calc_fsc_using_halfmaps(
            #    self.phrand_fhf_lst[i],self.phrand_fhf_lst[i+1],self.bin_idx,self.nbin,debug_mode,nx,ny,nz)
            full_fsc_noise = 2.0 * rbin_fsc / (1.0 + rbin_fsc)

            # fsc_true from Richard's formular
            fsc_true = (full_fsc_total - full_fsc_noise) / (1 - full_fsc_noise)
            # replace fsc_true with fsc_masked_full upto resol_rand_idx + 2 (RELION uses 2)
            fsc_true[:idx+2] = full_fsc_total[:idx+2] 

            # override fec_true with fsc_unmasked
            #fsc_true = full_fsc_unmasked
            fobj.write('\n')
            fobj.write(' All halfmap FSCs have been converted into fullmap FSC \n')
            fobj.write('\n')
            fobj.write(' bin# \n')
            fobj.write(' bin resolution (A) \n')
            fobj.write(' Unmasked FSC \n')
            fobj.write(' Masked FSC \n')
            fobj.write(' Randomized FSC \n')
            fobj.write(' True FSC \n')
            fobj.write('\n')
            for j in range(len(self.res_arr)):
                fobj.write("{:5d} {:6.2f} {:8.4f} {:8.4f} {:8.4f} {:8.4f}\n".format(
                            j,
                            self.res_arr[j],
                            full_fsc_unmasked[j],
                            full_fsc_total[j],
                            full_fsc_noise[j],
                            fsc_true[j]))

            # plot various FSCs
            fobj.write('\n FSC were plotted into fsc_%d.eps \n' %i)
            emda.plotter.plot_nlines(self.res_arr,
                                [full_fsc_unmasked,full_fsc_total,full_fsc_noise,fsc_true],
                                "{0}_{1}.{2}".format('fsc',str(i),'eps'),
                                ["unmasked","fsc_t","fsc_n","fsc_true"])
            
            fFo_lst.append(fo)
            fEo_lst.append(eo)
            fBNV_lst.append(noisevar)
            fBSV_lst.append(signalvar)
            fBTV_lst.append(totalvar)
            fBFsc_lst.append(fsc_true)
            #umfFo_lst.append(umfo)
            #umfEo_lst.append(umeo)
            #umfBNV_lst.append(umnoisevar)
            #umfBSV_lst.append(umsignalvar)
            #umfBTV_lst.append(umtotalvar)
            #umfBFsc_lst.append(umbin_fsc)
        #
        self.fo_lst            = fFo_lst
        self.eo_lst            = fEo_lst
        self.signalvar_lst     = fBSV_lst
        self.totalvar_lst      = fBTV_lst
        self.hffsc_lst         = fBFsc_lst
        #self.umfo_lst            = umfFo_lst
        #self.umeo_lst            = umfEo_lst
        #self.umsignalvar_lst     = umfBSV_lst
        #self.umtotalvar_lst      = umfBTV_lst
        #self.umhffsc_lst         = umfBFsc_lst



from __future__ import division, absolute_import, print_function
import setuptools
from numpy.distutils.core import setup, Extension

ex1 = Extension(name = 'fcodes_fast',
                sources = ['emda/fcodes_fast.f90'])

setup(name= 'emda',
    version= '1.1.2.post8',
    description= 'Electron Microscopy map and model manipulation tools',
    long_description='''\
        Python library for Electron Microscopy map and model
        manipulations. To work with MRC/CCP4.MAP and MTZ files. Map to map
        fitting and average/difference map calculation. Map and map-model validation
        using 3D correlations.''',
    url= None,
    author= 'Rangana Warshamanage, Garib N. Murshudov',
    author_email= 'ranganaw@mrc-lmb.cam.ac.uk, garib@mrc-lmb.cam.ac.uk',
    license= 'MPL-2.0',
    packages= ['emda','emda.mapfit'],
    ext_modules = [ex1],
    install_requires=['pandas==0.23.4','mrcfile','matplotlib','numpy','scipy'],
    dependency_links=['git+https://github.com/project-gemmi/gemmi.git'],
    test_suite='emda.tests',
    entry_points={
      'console_scripts': [
          'emda_test = emda.emda_test:main',
          'emda = emda.emda_cmd_caller:main',
                          ],
      },
    zip_safe= False)



from __future__ import absolute_import, division, print_function, unicode_literals

def plot_line_log(res_arr,arr):
    import matplotlib
    matplotlib.use('TkAgg')
    from matplotlib import pyplot as plt
    import numpy as np
    d2vec = []
    for i in range(len(res_arr)):
        d2vec.append(float(1/(res_arr[i])))
    fig,ax = plt.subplots(1,1)
    plt.plot(d2vec,np.log10(arr),"r")
    xmin = np.min(d2vec); xmax = np.max(d2vec)
    ax.set_xlim(xmin,xmax)
    ax.set_xticks(d2vec)
    str1 = []
    for e in res_arr:
        str1.append('{:.3}'.format(e))
    for n in range(len(str1)):
        if n % 5 != 0:
            str1[n] = " "
    ax.set_xticklabels(str1,rotation=90,fontsize=9)
    plt.xlabel('Resolution')
    plt.ylabel('log.Variance(Fo)')
    plt.show()

def plot_nlines_log(res_arr,
                    list_arr,
                    #curve_label=["Signal","Noise","Predicted"],
                    curve_label=None,
                    mapname=None
                    ):
    import matplotlib
    matplotlib.use('TkAgg') 
    from matplotlib import pyplot as plt
    import numpy as np
    if mapname is None: mapname='variances.eps'
    bin_arr = np.arange(len(res_arr))
    fig = plt.figure(figsize=(6,4))
    ax1 = fig.add_subplot(111)
    for icurve in range(len(list_arr)):
        data = list_arr[icurve]
        if curve_label is None:
            label = 'Set#'+str(icurve)
            ax1.plot(bin_arr,np.log10(data),label=label,linewidth=2)
        else:
            ax1.plot(bin_arr,np.log10(data),label=curve_label[icurve],linewidth=2)
    pos = np.array(ax1.get_xticks(), dtype=np.int)
    n_bins = res_arr.shape[0]
    pos[pos < 0] = 0
    pos[pos >= n_bins] = n_bins - 1
    ax1.set_xticklabels(np.round(res_arr[pos], decimals=2))
    ax1.set_xlabel("Resolution ($\AA$)")
    plt.ylabel('log.Variance(Fo)')
    font = {'family':'serif','color':'black','weight':'bold','size':16}
    plt.legend(loc=0)
    plt.savefig(mapname,format='eps',dpi=300)
    plt.show()

def plot_line(mapname,res_arr,arr):
    import matplotlib
    matplotlib.use('TkAgg')
    from matplotlib import pyplot as plt
    import numpy as np
    bin_arr = np.arange(len(res_arr))
    fig = plt.figure(figsize=(6,4))
    ax1 = fig.add_subplot(111)
    ax1.plot(bin_arr,arr,linewidth=2)
    xmin = np.min(bin_arr); xmax = np.max(bin_arr)
    #plt.plot((xmin, xmax), (0.143, 0.143), 'k--')
    pos = np.array(ax1.get_xticks(), dtype=np.int)
    n_bins = res_arr.shape[0]
    pos[pos < 0] = 0
    pos[pos >= n_bins] = n_bins - 1
    ax1.set_xticklabels(np.round(res_arr[pos], decimals=2))
    ax1.set_xlabel("Resolution ($\AA$)")
    plt.ylabel('Fourier Shell Correlation')
    font = {'family':'serif','color':'black','weight':'bold','size':16}
    plt.savefig(mapname,format='eps',dpi=300)
    plt.show()

def plot_nlines(res_arr,
                list_arr,
                mapname='halfmap_fsc.eps',
                curve_label=["halfmap_fsc"]
                ):
    import matplotlib
    matplotlib.use('TkAgg')
    from matplotlib import pyplot as plt
    import numpy as np
    from mpl_toolkits.axes_grid1 import host_subplot
    import mpl_toolkits.axisartist as AA
    bin_arr = np.arange(len(res_arr))
    fig = plt.figure(figsize=(6,4))
    #ax1 = host_subplot(111,axes_class=AA.Axes)
    ax1 = fig.add_subplot(111)
    for icurve in range(len(list_arr)):
        #ax1.plot(d2vec,list_arr[icurve],label=str(icurve),linewidth=2)
        ax1.plot(bin_arr,list_arr[icurve],label=curve_label[icurve],linewidth=2)
    xmin = np.min(bin_arr); xmax = np.max(bin_arr)
    plt.plot((xmin, xmax), (0.143, 0.143), color='gray',linestyle=':')
    plt.plot((xmin, xmax), (0.0, 0.0), color='black',linestyle=':')
    pos = np.array(ax1.get_xticks(), dtype=np.int)
    n_bins = res_arr.shape[0] #data.shape[0]
    pos[pos < 0] = 0
    pos[pos >= n_bins] = n_bins - 1
    ax1.set_xticklabels(np.round(res_arr[pos], decimals=2))
    ax1.set_xlabel("Resolution ($\AA$)")
    plt.legend(loc=0)
    plt.ylabel('Fourier Shell Correlation')
    font = {'family':'serif','color':'black','weight':'bold','size':16}
    plt.savefig(mapname,format='eps',dpi=300)

def plot_nlines2(res_arr,
                list_arr,
                mapname='halfmap_fsc.eps',
                curve_label=["halfmap_fsc"]
                ):
    import matplotlib
    matplotlib.use('TkAgg')
    from matplotlib import pyplot as plt
    import numpy as np
    from mpl_toolkits.axes_grid1 import host_subplot
    import mpl_toolkits.axisartist as AA
    bin_arr = np.arange(len(res_arr))
    fig = plt.figure(figsize=(6,4))
    #ax1 = host_subplot(111,axes_class=AA.Axes)
    ax1 = fig.add_subplot(111)
    for icurve in range(len(list_arr)):
        #ax1.plot(d2vec,list_arr[icurve],label=str(icurve),linewidth=2)
        ax1.plot(bin_arr,list_arr[icurve],label=curve_label[icurve],linewidth=2)
    xmin = np.min(bin_arr); xmax = np.max(bin_arr)
    plt.plot((xmin, xmax), (0.5, 0.5), 'k--')

    pos = np.array(ax1.get_xticks(), dtype=np.int)
    n_bins = res_arr.shape[0] #data.shape[0]
    pos[pos < 0] = 0
    pos[pos >= n_bins] = n_bins - 1
    ax1.set_xticklabels(np.round(res_arr[pos], decimals=2))
    ax1.set_xlabel("Resolution (1/$\AA$)")
    plt.legend(loc=0)
    plt.ylabel('Fourier Shell Correlation')
    font = {'family':'serif','color':'black','weight':'bold','size':16}
    plt.savefig(mapname,format='eps',dpi=300)

def plot_3d(data):
    import matplotlib
    matplotlib.use('TkAgg')
    from matplotlib import pyplot as plt
    from mpl_toolkits.mplot3d import Axes3D # implicit import
    import numpy as np
    fig = plt.figure()
    ax = fig.gca(projection='3d')
    #nan_mask = Z > 0
    r = len(data)
    x_dim = r
    y_dim = r
    X = np.linspace(1,x_dim,x_dim)
    Y = np.linspace(1,y_dim,y_dim)
    ax.scatter(X,Y,data, marker='.', color='r', s=3)
    plt.xlabel('X')
    plt.ylabel('Y')
    ax.set_zlabel('Z')
    ax.axis('equal')
    ax.axis('tight')
    plt.show()

def plot3dnonzero(data3d):
    dim = data3d.shape[0]
    z,x,y = data3d.nonzero()
    import matplotlib
    matplotlib.use('TkAgg')
    import matplotlib.pyplot as plt
    import numpy as np
    from mpl_toolkits.mplot3d import Axes3D
    fig = plt.figure()
    ax = fig.add_subplot(111, projection='3d')
    #x = np.arange(-x/2, x/2)
    #y = np.arange(-y/2, y/2)
    ax.scatter(x, y, -z, zdir='z', c= 'red',marker='.')
    plt.show()

def plot4dcolor(data):   
    import matplotlib
    matplotlib.use('TkAgg')
    from mpl_toolkits.mplot3d import Axes3D
    import matplotlib.pyplot as plt
    import numpy as np
    ih,ik,il = data.shape
    h = np.arange(-ih/2,ih/2); k = np.arange(-ik/2,ik/2); l = np.arange(-il/2,il/2)
    y,x,z = np.meshgrid(k,h,l)
    c = data.flatten()
    fig = plt.figure()
    ax = fig.add_subplot(111, projection='3d')
    ax.scatter(x, y, z, c=c, cmap=plt.hot())
    #ax.scatter(x, y, z, c, color='b')
    plt.show()

def contour_nplot(nx,ny,lst_data):
    import matplotlib
    matplotlib.use('TkAgg')
    import matplotlib.pyplot as plt
    plt.style.use('seaborn-white')
    import numpy as np
    plt.figure(figsize=(30,10))
    x = np.linspace(0, nx, nx)
    y = np.linspace(0, ny, ny)
    X, Y = np.meshgrid(x, y)
    lst_minval = []
    lst_maxval = []
    for imap in lst_data:
        lst_minval.append(np.amin(imap))
        lst_maxval.append(np.amax(imap))
    min_val = min(lst_minval)
    max_val = max(lst_maxval)
    i = 0
    n = len(lst_data)
    #fig = plt.figure(figsize=(30,10))
    from mpl_toolkits.axes_grid1 import make_axes_locatable
    fig, (ax1, ax2, ax3) = plt.subplots(ncols=3)
    for imap in lst_data:
        i = i + 1
        plt.subplot(1,n,i)
        if i == 3:
            img1 = plt.contourf(X, Y, imap, 200, cmap='RdGy', levels=np.linspace(min_val,max_val,20))
            divider = make_axes_locatable(ax3)
            cax1 = divider.append_axes("right", size="5%", pad=0.05)
            fig.colorbar(img1, cax=cax1)
        else:
            plt.contourf(X, Y, imap, 200, cmap='RdGy', levels=np.linspace(min_val,max_val,20))
        #plt.xlim(0,60)
        #plt.ylim(0,60)
    #plt.colorbar()

    '''min_val1 = np.amin(data1)
    min_val2 = np.amin(data2)
    min_val = min([min_val1,min_val2])
    max_val1 = np.amax(data1)
    max_val2 = np.amax(data2)
    max_val = max([max_val1,max_val2])   
    plt.subplot(1,2,1)
    #plt.imshow(data1)
    plt.contourf(X, Y, data1, 200, cmap='RdGy', levels=np.linspace(min_val,max_val,20))
    #plt.contourf(X, Y, data, 200, cmap=plt.cm.bone)
    plt.colorbar()
    plt.axis('equal')
    #plt.show()
    plt.subplot(1,2,2)
    plt.contourf(X, Y, data2, 200, cmap='RdGy', levels=np.linspace(min_val,max_val,20))
    #plt.contourf(X, Y, data, 200, cmap=plt.cm.bone)
    plt.colorbar()
    plt.axis('equal')'''
    #plt.subplots_adjust(wspace=0,hspace=0)
    plt.show()

def contour_nplot2(data_3d):
    import matplotlib
    matplotlib.use('TkAgg')
    import matplotlib.pyplot as plt
    plt.style.use('seaborn-white')
    import numpy as np
    #plt.figure(figsize=(10,10))
    nx, ny, nz = data_3d.shape
    
    x = np.linspace(0, nx, nx)
    y = np.linspace(0, ny, ny)
    X, Y = np.meshgrid(x, y)
    min_val = np.amin(data_3d)
    max_val = np.amax(data_3d)
    #from mpl_toolkits.axes_grid1 import make_axes_locatable
    #fig, (ax1, ax2, ax3) = plt.subplots(ncols=1)
    for i in range(data_3d.shape[2]):
        plt.title('Z= %i' % i)
        plt.contourf(X, Y,data_3d[:,:,i] , 200, cmap='RdGy', levels=np.linspace(min_val,max_val,20))
        plt.axis('equal')
        plt.show()

def plot_from_twofiles_csv(filename,labels):
    import pandas
    import numpy as np
    import matplotlib
    matplotlib.use('TkAgg')
    data_refmac = pandas.read_csv(filename[0],delimiter=' ')
    data_other = pandas.read_csv(filename[1],delimiter=' ')
    bf_arr = abs(data_other[labels[0]])
    blur_cc_other = data_other[labels[1]]
    blur_cc_refmac = data_refmac[labels[1]]
    #bf_arr = np.array([0,10,20,30,40,50,60,70,80,90,100,120,140,160])
    from matplotlib import pyplot as plt
    fig = plt.figure(figsize=(6,4))
    ax1 = fig.add_subplot(111)
    ax1.plot(bf_arr,blur_cc_other,label='Other',linewidth=2)
    ax1.plot(bf_arr,blur_cc_refmac,label='Refmac',linewidth=2)
    xmin = np.min(bf_arr); xmax = np.max(bf_arr)
    plt.plot((xmin, xmax), (1.0, 1.0), 'k--')
    ax1.set_xlabel("B factor ($\AA^2$)")
    plt.legend(loc=0)
    plt.ylabel('Overall Correlation Coefficient')
    plt.savefig('overall_CC_both.eps',format='eps',dpi=300)
    plt.show()



# calcuating bestmap - normalized and FSC weighted map
def bestmap(f1, f2, bin_idx, nbin, mode=1, kernel_size=5):
    if mode==1:
        print('bestmap is calculated using FSC in resolution bins')
        bestmap = bestmap_1dfsc(f1, f2, bin_idx, nbin)
    elif mode==2:
        print('bestmap is calculated using local FSC')
        bestmap = bestmap_3dfsc(f1, f2, kernel_size)
    return bestmap

def bestmap_1dfsc(f1, f2, bin_idx, nbin):
    import fcodes_fast
    from emda import fsc
    import numpy as np
    cx,cy,cz = f1.shape
    fsc,_,_,_,_,eo = fsc.halfmaps_fsc_variance(f1, f2, bin_idx, nbin)
    fsc_grid = fcodes_fast.read_into_grid(bin_idx,
                            fsc,
                            nbin,
                            cx,cy,cz)
    fsc_grid_filtered = np.where(fsc_grid < 0.0, 0.0, fsc_grid)
    return np.sqrt(fsc_grid_filtered) * eo

def bestmap_3dfsc(f1, f2, kernel_size=5):
    import numpy as np
    from emda.restools import get_resArr,remove_edge,create_soft_edged_kernel_pxl
    from emda.fouriersp_corr_3d import get_3dnoise_variance,get_3dtotal_variance
    kernel = create_soft_edged_kernel_pxl(kernel_size)
    # calculate 3D fourier correlation
    noise = get_3dnoise_variance(f1, f2, kernel)
    total = get_3dtotal_variance(f1, f2, kernel)
    total_pos = np.where(total <= 0., 0.1, total)
    fsc = (total - noise) / total_pos
    fsc_grid_filtered = np.where((total - noise) <= 0., 0., fsc)
    eo = (f1 + f2) / (2 * np.sqrt(total_pos))
    eo = np.where(total <= 0., 0., eo)
    return np.sqrt(fsc_grid_filtered) * eo


"""
Author: "Rangana Warshamanage, Garib N. Murshudov"
MRC Laboratory of Molecular Biology
    
This software is released under the
Mozilla Public License, version 2.0; see LICENSE.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

import numpy as np
from emda.config import *
#debug_mode = 1

def get_resolution_array(uc,hf1):
    import fcodes_fast
    nx,ny,nz = hf1.shape
    maxbin = np.amax(np.array([nx//2,ny//2,nz//2]))
    nbin,res_arr,bin_idx,sgrid = fcodes_fast.resolution_grid(uc,debug_mode,maxbin,nx,ny,nz)
    return nbin,res_arr[:nbin],bin_idx

def get_resArr(unit_cell,nx):
    import numpy as np
    # Generate resolution array
    a,b,c = unit_cell[:3]
    narr = np.arange(nx//2)
    narr[0] = 1.0
    fResArr = a / narr
    fResArr[0] = 5000
    #for i in range(nx//2):
    #    print(i,fResArr[i])
    return fResArr

def create_kernel(fResArr, smax):
    # Create kernel. smax is resolution to which the kernel
    # is defined.
    import numpy as np
    dist = np.sqrt((fResArr - smax)**2)
    cbin = np.argmin(dist)
    print('cbin',cbin)
    box_size = 2 * cbin + 1
    box_radius = cbin + 1
    # Box mask
    #kern_3d = np.ones((box_size, box_size, box_size)) / (box_size ** 3)
    #mask = kern_3d
    # Creating a sphere mask (binary mask)
    center = [box_radius-1, box_radius-1, box_radius-1]
    print('boxsize: ',box_size,'boxradius: ',box_radius,'center:',center)
    radius = box_radius
    X, Y, Z = np.ogrid[:box_size, :box_size, :box_size]
    dist_from_center = np.sqrt((X - center[0])**2 + (Y-center[1])**2 + (Z-center[2])**2)
    mask = dist_from_center <= radius
    return mask

def create_soft_edged_kernel(fResArr,smax):
    # Create soft-edged-kernel. smax is resolution to which the kernel size
    # is defined
    import numpy as np
    mask = create_kernel(fResArr,smax)
    kern_sphere = mask/np.sum(mask) # Normalized mask
    #norm = 1/np.sum(mask)
    print('Number of data points inside kernel: ', np.count_nonzero(mask))
    # Creating soft-edged mask - Naive approach
    from math import sqrt,cos,sin
    kern_sphere_soft = np.zeros(shape=(kern_sphere.shape),dtype='float')
    r1 = kern_sphere.shape[0]//2 + 1
    r0 = r1 - int(round(r1 * 0.3))
    print('r1: ', r1, 'r0: ', r0)
    #print('norm: ', norm)
    kx = kern_sphere.shape[0]
    ky = kern_sphere.shape[1]
    kz = kern_sphere.shape[2]
    center = [r1-1, r1-1, r1-1]
    for i in range(kx):
        for j in range(ky):
            for k in range(kz):
                dist = sqrt((i - center[0])**2 + (j - center[0])**2 + (k - center[0])**2)
                if dist <= r1:
                    if dist < r0:
                        #kern_sphere_soft[i,j,k] = norm
                        kern_sphere_soft[i,j,k] = 1
                    else:
                        #kern_sphere_soft[i,j,k] = (norm * (1 + cos(np.pi * (dist - r0)/(r1 - r0))))/2.0
                        kern_sphere_soft[i,j,k] = ((1 + cos(np.pi * (dist - r0)/(r1 - r0))))/2.0
    kern_sphere_soft = kern_sphere_soft / np.sum(kern_sphere_soft)
    return kern_sphere_soft

def create_soft_edged_kernel_pxl(r1):
    # Create soft-edged-kernel. r1 is the radius of kernel in pixels
    from math import sqrt,cos,sin
    import numpy as np
    #from plotter import contour_nplot2
    if r1 < 3: r1 = 3
    if r1%2 == 0: r1 = r1 - 1
    boxsize = 2 * r1 + 1
    kern_sphere_soft = np.zeros(shape=(boxsize,boxsize,boxsize),dtype='float')
    kx = ky = kz = boxsize
    center = boxsize//2
    print('center: ', center)
    r1 = center
    r0 = r1 - 2
    print('r1: ', r1, 'r0: ', r0)
    for i in range(kx):
        for j in range(ky):
            for k in range(kz):
                dist = sqrt((i - center)**2 + (j - center)**2 + (k - center)**2)
                if dist < r1:
                    if dist < r0:
                        kern_sphere_soft[i,j,k] = 1
                    else:
                        kern_sphere_soft[i,j,k] = ((1 + cos(np.pi * (dist - r0)/(r1 - r0))))/2.0
                #print(i,j,k,r1,r0,dist,kern_sphere_soft[i,j,k])
    kern_sphere_soft = kern_sphere_soft / np.sum(kern_sphere_soft)
    #contour_nplot2(kern_sphere_soft)
    return kern_sphere_soft

def remove_edge(fResArr, smax):
    # Remove everything outside smax-resolution.
    import numpy as np
    dist = np.sqrt((fResArr - smax)**2)
    cbin = np.argmin(dist)
    box_radius = cbin + 1
    box_size = cbin * 2 + 1
    # Box mask
    #kern_3d = np.ones((box_size, box_size, box_size)) / (box_size ** 3)
    #mask = kern_3d
    # Creating a sphere mask
    center = [box_radius, box_radius, box_radius]
    print('boxsize: ',box_size,'boxradius: ',box_radius,'center:',center)
    radius = box_radius
    X, Y, Z = np.ogrid[:box_size, :box_size, :box_size]
    dist_from_center = np.sqrt((X - center[0])**2 + (Y-center[1])**2 + (Z-center[2])**2)
    mask = dist_from_center <= radius
    return mask

def cut_resolution(f,bin_idx,res_arr,cbin):
    import fcodes_fast
    # Making data for map fitting
    nx,ny,nz = f.shape
    '''dist = np.sqrt((res_arr - smax)**2)
    cbin = np.argmin(dist) + 1 # adding 1 because fResArr starts with zero
    print('cnbin=', cbin)'''
    # fcodes.cutmap cuts f according to resolution defined by smax and output same size
    # map as f but padded with zeros outside smax
    fout = fcodes_fast.cutmap(f,bin_idx,cbin,0,len(res_arr),nx,ny,nz)
    # cutmapresize.cutmap_resize imposes smax on f and resizes mapsize
    #import cutmapresize
    #fout = cutmapresize.cutmap_resize(f,bin_idx,cbin,0,len(res_arr),nx,ny,nz)
    return fout

def cut_resolution_for_linefit(f,bin_idx,res_arr,smax):
    import fcodes_fast
    import numpy as np
    # Making data for map fitting
    nx,ny,nz = f.shape
    '''dist = np.sqrt((res_arr - smax)**2)
    cbin = np.argmin(dist) #+ 1 # adding 1 because fResArr starts with zero
    if cbin%2 != 0: cx = cbin + 1
    else: cx = cbin
    if not cbin <= bin_idx.shape[0]//2:
        print('Max allowed dim: ', bin_idx.shape[0]//2)
        print('Current dim: ', cbin)
        exit()'''
    cbin = cx = smax
    dx = int((nx - 2*cx)/2)
    dy = int((ny - 2*cx)/2)
    dz = int((nz - 2*cx)/2)
    cBIdx = bin_idx[dx:dx+2*cx,dy:dy+2*cx,dz:dz+2*cx]
    # fcodes.cutmap cuts f according to resolution defined by smax and output same size
    # map as f but padded with zeros outside smax
    fout = fcodes_fast.cutmap(f,bin_idx,cbin,0,len(res_arr),nx,ny,nz)[dx:dx+2*cx,dy:dy+2*cx,dz:dz+2*cx]
    return fout,cBIdx,cbin






def cc_overall_realsp(map1, map2):
    # Overall correlation - real space
    import numpy as np
    import math
    mean1 = np.mean(map1)
    mean2 = np.mean(map2)
    covar = np.sum(map1 * map2 - (mean1 * mean2))
    var1 = np.sum(map1 * map1 - (mean1 * mean1))
    var2 = np.sum(map2 * map2 - (mean2 * mean2))
    occ = covar / math.sqrt(var1 * var2)
    return 2.0 * occ / (1.0 + occ)

def cc_overall_fouriersp(f1, f2):
    import numpy as np
    import math
    f1_mean = np.mean(f1)
    f2_mean = np.mean(f2)
    covar = np.sum(f1 * np.conjugate(f2) - f1_mean * np.conjugate(f2_mean))
    var1 = np.sum(f1 * np.conjugate(f1) - f1_mean * np.conjugate(f1_mean))
    var2 = np.sum(f2* np.conjugate(f2) - f2_mean * np.conjugate(f2_mean))
    fsc = (covar.real / math.sqrt(var1.real * var2.real))
    return 2.0 * fsc / (1.0 + fsc)



"""
Author: "Rangana Warshamanage, Garib N. Murshudov"
MRC Laboratory of Molecular Biology
    
This software is released under the
Mozilla Public License, version 2.0; see LICENSE.
"""

from __future__ import absolute_import, division, print_function, unicode_literals

# rotation matrix to quaternion
# http://www.euclideanspace.com/maths/geometry/rotations/conversions/matrixToQuaternion/index.htm

#assumes the upper 3x3 of m is a pure rotation matrix (i.e, unscaled)

def rot2quart(rm):
    import math
    import numpy as np

    m11 = rm[ 0,0 ]; m12 = rm[ 0,1 ]; m13 = rm[ 0,2 ]
    m21 = rm[ 1,0 ]; m22 = rm[ 1,1 ]; m23 = rm[ 1,2 ]
    m31 = rm[ 2,0 ]; m32 = rm[ 2,1 ]; m33 = rm[ 2,2 ]

    trace = m11 + m22 + m33

    q = np.zeros(4, dtype=np.float64)

    if ( trace > 0 ):
    	s = 0.5 / math.sqrt( trace + 1.0 )
    	q[0] = 0.25 / s
    	q[1] = ( m32 - m23 ) * s
    	q[2] = ( m13 - m31 ) * s
    	q[3] = ( m21 - m12 ) * s
    elif m11 > m22 and m11 > m33:
    	s = 2.0 * math.sqrt( 1.0 + m11 - m22 - m33 )
    	q[0] = ( m32 - m23 ) / s
    	q[1] = 0.25 * s
    	q[2] = ( m12 + m21 ) / s
    	q[3] = ( m13 + m31 ) / s
    elif m22 > m33:
    	s = 2.0 * math.sqrt( 1.0 + m22 - m11 - m33 )
    	q[0] = ( m13 - m31 ) / s
    	q[1] = ( m12 + m21 ) / s
    	q[2] = 0.25 * s
    	q[3] = ( m23 + m32 ) / s
    else:
    	s = 2.0 * math.sqrt( 1.0 + m33 - m11 - m22 )
    	q[0] = ( m21 - m12 ) / s
    	q[1] = ( m13 + m31 ) / s
    	q[2] = ( m23 + m32 ) / s
    	q[3] = 0.25 * s
    return q

if (__name__ == "__main__"):
    import numpy as np
    from quaternions import get_RM,get_quaternion

    #q_init = np.array([1.0, 0.0, 0.0, 0.0], dtype=np.float64)
    theta = [(0.5,0.5,0),180.0]
    q_init = get_quaternion(theta)
    print(q_init)
    q = rot2quart(get_RM(q_init))
    print(q)


# difference map 
from emda import iotools
from emda.mapfit import mapaverage
import numpy as np
import fcodes_fast
from emda.config import *

def difference_map(f1, f2, cell, origin=None, res_arr=None, bin_idx=None, nbin=None, s_grid=None):
    from emda import fsc, restools
    nx, ny, nz = f1.shape
    if s_grid is None:
        maxbin = np.amax(np.array([nx//2,ny//2,nz//2]))
        _, s_grid, _ = fcodes_fast.resolution_grid_full(cell,0.0,1,maxbin,nx,ny,nz)
    if bin_idx is None:
        nbin,res_arr,bin_idx = restools.get_resolution_array(cell,f1)
    # estimating covariance between current map vs. static map
    f1f2_fsc,_ = fsc.anytwomaps_fsc_covariance(f1,f2,bin_idx,nbin)
    '''_,f1f2_fsc = fcodes_fast.calc_covar_and_fsc_betwn_anytwomaps(
                            f1,f2,bin_idx,nbin,0,nx,ny,nz)'''
    # find the relative B-factor between static and frt_full
    resol = mapaverage.get_resol(f1f2_fsc,res_arr)
    scale, bfac_relative = mapaverage.estimate_scale_and_relativebfac([f1,f2],cell,resol)
    #fobj.write('Relative B-factor between static map and moving(full) map: '+ str(bfac_relative)+'\n')
    # apply B-factor and scale to 2nd map
    f2_scaled = scale * f2 * np.exp(bfac_relative * (s_grid**2)/4.0)
    # calculating maps
    dm1_dm2 = np.real(np.fft.ifftshift(
                         np.fft.ifftn(
                         np.fft.ifftshift(f1 - f2_scaled))))
    dm2_dm1 = np.real(np.fft.ifftshift(
                         np.fft.ifftn(
                         np.fft.ifftshift(f2_scaled - f1))))
    iotools.write_mrc(dm1_dm2,
                      'diffmap_m1-m2.mrc',
                      cell, origin)
    iotools.write_mrc(dm2_dm1,
                      'diffmap_m2-m1.mrc',
                      cell, origin)